`timescale 1ps / 1ps
//module 'serial_io' generates patterns for SPI signals, reads serial LTC2348 data and converts it to parallel using Altera CycloneIII
//fix timing issue that prevented all channels from clocking out
//GMH
//December 4, 2015  RevB
module serial_io(master_clk, mosi, sck, conv, aux0, mcd, B16_18L, WRIN, mosi_par, d_latch);	 
   input master_clk;	   								// external clock in
   input mcd;    											// mcd is a delayed version of master_clk
	input B16_18L;											// 1=16bits, 0=18bits
	input WRIN;												// resets serial_io when WRIN goes low
   output mosi;			  								// sdi to adc 
   
	
	input [23:0] mosi_par;								// config word from DC890
	
	output d_latch;										// starts data_latch period
   
   output sck;												// gated version of external clock in with some delay
   output conv;											// convert or CS to ADC
   output aux0;  											// aux output   
   					
   reg [39:0] aux0_shift;
   reg [39:0] conv_shift;
   reg [39:0] data_latch_shift;
   reg [39:0] delay_shift;												
   reg [39:0] mosi_shift;
   reg [39:0] sck_shift;
	
   
	wire [23:0] mosi_par;
	
   reg [11:0] delay;
   reg [6:0] clk_counter;
   wire[11:0] delay_time;
   wire[6:0] max_clk_cnt;
  
   wire aux0;		 
   
	wire d_latch;											//added 11/21/13 (advances next sdox to dout. last one ignored)
	
   wire master_clk;
	
   wire mcd;
	wire B16_18L;
	wire WRIN;
   
   wire mosi;
   wire sck;
	
	reg [39:0]miso_def;
	wire [39:0]mosi_def;
	reg [39:0]data_latch_def;
	reg [39:0]sck_def;
	reg [39:0]conv_def;
	reg [39:0]aux0_def;
	reg [39:0]delay_def;

	
	

	
	always @( B16_18L )begin
	if (B16_18L==0) begin
																	//  up to 4 16bit channels in 64bits. Increase to 144 to do 8 channels of 18bits
																	//  counts per delay	 		max clk count 		# of channels 	# of bits 
																	//   b16-b23(max 4095)	   b9-b15(max 127)	b5-b8(max 16)	b0-b4 (max 32)
																	//		264								37			         1		  		   18	   
          miso_def <= 40'H0001074831;    				//  0001 0000 0111				0100 	100			0 001 		 	1 0001   	                               												      
 //         mosi_def <= 40'H0000000000 & mosi_par; //  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000        	                               
    data_latch_def <= 40'H07F8000000;    				//  0001 1111 1111 1000 0000 0000 0000 0000 0000 0000	   	           	                               
           sck_def <= 40'H0007FFFFF8;    				//  0000 0000 0000 0111 1111 1111 1111 1111 1111 1000    	    	                               
          conv_def <= 40'HFFFFFFFFFC;    				//  1111 1111 1111 1111 1111 1111 1111 1111 1111 1100 	   	    	          	                           
          aux0_def <= 40'H0000000001;					//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0001
         delay_def <= 40'H0000000002;      			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0010 

	end	    
				else begin   

																	//  up to 4 16bit channels in 64bits. Increase to 144 to do 8 channels of 18bits
																	//  counts per delay	 		max clk count 		# of channels 	# of bits 
																	//   b16-b23(max 255)	   b9-b15(max 127)	b5-b8(max 16)	b0-b4 (max 32)
																	//		264									37			    1		  		   18	   
          miso_def <= 40'H0001074831;    				//  0001 0000 0111				0100 	100			0 001 		 	1 0001   	                               												      
  //        mosi_def <= 40'H0000000000;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000        	                               
    data_latch_def <= 40'H07F8000000;    				//  0001 1111 1111 1000 0000 0000 0000 0000 0000 0000	   	           	                               
           sck_def <= 40'H0007FFFFF8;    				//  0000 0000 0000 0111 1111 1111 1111 1111 1111 1000    	    	                               
          conv_def <= 40'HFFFFFFFFFC;    				//  1111 1111 1111 1111 1111 1111 1111 1111 1111 1100 	   	    	          	                           
          aux0_def <= 40'H0000000001;					//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0001
         delay_def <= 40'H0000000002;      			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0010 

				end
    end
																	// CLKIN is in phase with sck
																	// mcd is a delayed version of CLKIN
																	// SCKO changes state on rising edge sck
	assign mosi_def = 40'H0000000000 + {mosi_par,3'b000};		//added 2/11/14
	assign max_clk_cnt =  miso_def[15:9];				// maximum count before clk_counter is reset	(max 127)   
		  
	assign delay_time =  miso_def[27:16];				// number of master_clks to wait if delay goes high (max 255)
	assign d_latch = (data_latch_shift[0]& mcd);		//modified 11/21/13 was data_latch=(data_latch_shift[0]&mcd)
		
	assign mosi = mosi_shift[0];						
	assign aux0 = aux0_shift[0];
	assign sck = (sck_shift[0]& mcd);					//was master_clk		mcd gives proper delay
	
	wire conv;
	assign	conv = !(conv_shift[0]);												
   initial begin //
   delay <= 0;															
   clk_counter <= 7'b0000000;								// initialize counter loop   
   aux0_shift<=       40'H000000000;
   conv_shift<=       40'H000000000;
   data_latch_shift<= 40'H000000000;
   delay_shift<=      40'H000000000;
   mosi_shift<=       40'H000000000;
   sck_shift<=        40'H000000000;
	
   
	
   end //
 
always @(negedge master_clk or negedge WRIN)	begin	// everything happens on neg edge of master clk or WRIN
if (WRIN==0) begin//											//if WRIN goes low reset clk_counter
	clk_counter <= 7'b0000000;								//     2/13/14  was 7'b0000000
	data_latch_shift <= data_latch_def;						//
	conv_shift <= conv_def;											//
	mosi_shift <= mosi_def;												// added 5/7/14 to force reset when WRIN goes low
	sck_shift <= sck_def;											//
	aux0_shift <= aux0_def;										//
	delay_shift <= delay_def;								//
	delay <= 0;												//
end//
else begin//
if (delay_shift[0]==1)begin///	 										
	if (delay<delay_time)begin	////									
		delay <= delay + 12'b000000000001; 											
	end	////															
	else begin////													
		if(delay == delay_time) begin/////							
			clk_counter <= clk_counter- 7'b0000001;							
			data_latch_shift <= data_latch_shift >> 1;				
		   conv_shift <= conv_shift >> 1 ;							
			mosi_shift <= mosi_shift >> 1;							
			sck_shift <= sck_shift >> 1;							
			aux0_shift <= aux0_shift >> 1;							
			delay_shift <= delay_shift >> 1;	
		
		   delay <= 0;												
		end /////													
	end ////															
end///																
else begin///														
	if (clk_counter > 0) begin ////		  							
		clk_counter <= clk_counter - 7'b0000001;		  						
		data_latch_shift <= data_latch_shift >> 1;
		conv_shift <= conv_shift >> 1 ;
		mosi_shift <= mosi_shift >> 1;
		sck_shift <= sck_shift >> 1;
		aux0_shift <= aux0_shift >> 1;
		delay_shift <= delay_shift >> 1;
		
	 end ////	
	 else begin ////
		    clk_counter <= max_clk_cnt;			  		//if clk_counter=0 reset clk_counter 
			data_latch_shift <= data_latch_def;						
			conv_shift <= conv_def;
			mosi_shift <= mosi_def;
			sck_shift <= sck_def;
			aux0_shift <= aux0_def;
			delay_shift <= delay_def;
	
		
	end ////
end///
end//
end

endmodule